from ruamel.yaml import YAML
import os.path
yaml = YAML(typ='safe')
PACKAGE_DIR = os.path.dirname(__file__)
DATA_ROOT = '.'

def set_data_root(path):
    """
    Set the root directory of the data repository to be used with PINT Pal.
    PINT Pal will search this directory for a configuration file specifying settings
    such as the appropriate JPL ephemeris and version of TT(BIPM) to check for when
    validating timing models.

    It will also be treated as the base directory when resolving paths in YAML
    configuration files. This allows notebooks (or scripts) using YAML files within
    the data repository, which specify paths relative to the data root, to be run
    from other locations.

    The default value of `data_root` is '.' (the current working directory), which
    is sufficient in cases where either (1) no data repository is in use, or
    (2) all scripts and notebooks are run from the root of the data repository.
    """
    global DATA_ROOT
    DATA_ROOT = os.path.realpath(os.path.expanduser(path))
    try:
        read_config_file(os.path.join(DATA_ROOT, 'pint_pal_project.yaml'))
    except FileNotFoundError:
        pass

def reset_data_root():
    """
    Reset the data root and config variables to the default values.
    """
    global DATA_ROOT
    DATA_ROOT = '.'
    read_config_file(os.path.join(PACKAGE_DIR, 'defaults.yaml'))

def read_config_file(config_file):
    """
    Read a configuration file, along the lines of `defaults.yaml`, and load the results
    into a location that can be accessed by other PINT Pal code.
    """
    with open(config_file, 'r') as f:
        config = yaml.load(f)

    global LATEST_BIPM
    global LATEST_EPHEM
    global PLANET_SHAPIRO
    global CORRECT_TROPOSPHERE
    global FREQUENCY_RATIO
    global MAX_SOLARWIND_DELAY
    global LATEST_TOA_RELEASE

    if 'LATEST_BIPM' in config:
        LATEST_BIPM = config['LATEST_BIPM']
    if 'LATEST_EPHEM' in config:
        LATEST_EPHEM = config['LATEST_EPHEM']
    if 'PLANET_SHAPIRO' in config:
        PLANET_SHAPIRO = config['PLANET_SHAPIRO']
    if 'CORRECT_TROPOSPHERE' in config:
        CORRECT_TROPOSPHERE = config['CORRECT_TROPOSPHERE']
    if 'FREQUENCY_RATIO' in config:
        FREQUENCY_RATIO = config['FREQUENCY_RATIO']
    if 'MAX_SOLARWIND_DELAY' in config:
        MAX_SOLARWIND_DELAY = config['MAX_SOLARWIND_DELAY']
    if 'LATEST_TOA_RELEASE' in config:
        LATEST_TOA_RELEASE = config['LATEST_TOA_RELEASE']

read_config_file(os.path.join(PACKAGE_DIR, 'defaults.yaml'))



from os import makedirs
from os.path import dirname, join, split, splitext
from datetime import datetime
from glob import glob
import pytest
import pint_pal
from pint_pal.notebook_runner import run_template_notebook
test_dir = dirname(__file__)
base_dir = dirname(test_dir)
pint_pal.set_data_root(test_dir)

def config_files():
    config_files = (glob(join(test_dir, 'configs', 'B*.nb.yaml'))
                     + glob(join(test_dir, 'configs', 'J*.nb.yaml'))
                     + glob(join(test_dir, 'configs', 'B*.wb.yaml'))
                     + glob(join(test_dir, 'configs', 'J*.wb.yaml')))
    config_files = sorted(config_files)
    basenames = [splitext(split(filename)[1])[0] for filename in config_files]
    print(config_files)
    print(basenames)
    return [pytest.param(filename, id=basename) for filename, basename in zip(config_files, basenames)]

@pytest.fixture(scope='session', autouse=True)
def output_dir():
    output_dir = datetime.strftime(datetime.now(), 'tmp-%Y-%m-%dT%H-%M-%S')
    output_dir = join(base_dir, output_dir)
    makedirs(output_dir, exist_ok=True)
    return output_dir

@pytest.mark.parametrize('config_file', config_files())
def test_run_notebook(config_file, output_dir):
    """
    Run through the functions called in the notebook for each pulsar (excluding plotting).
    This will create a global log called test-run-notebooks.log, and a log file for each pulsar.

    To run for only one pulsar (using J1713+0747 as an example):
        `pytest tests/test_run_notebook.py::test_run_notebook[J1713+0747.nb]`
        or `pytest -k J1713+0747` (selects tests whose name contains "J1713+0747")
    To run for all pulsars in parallel (requires `pytest-xdist`):
        `pytest -n <workers> tests/test_run_notebook.py`
        <workers> is the number of worker processes to launch (e.g. 4 to use 4 CPU threads)
    """
    pint_pal.set_data_root(dirname(__file__))
    global_log = join(output_dir, f'test-run-notebook.log')
    with open(global_log, 'a') as f:
        run_template_notebook(
            'process_v1.2.ipynb',
            config_file,
            output_dir=output_dir,
            log_status_to=f,
            transformations = {
                'write_prenoise': "True",
                'write_results': "True",
                'use_existing_noise_dir': "True",
                'log_to_file': "True",
            },
        )
